--[[
Author: Lazy E Modding, Mantrid
https://www.lazyemodding.com
https://fs.mantrid.net/

Copyright: Ask Premission
]]


-- Some basic variables
BaleTextureChangerMenu = {
	CONTROLS = { -- The option boxes on the page
		STRAW125 = "multiRound125Straw",
		STRAW150 = "multiRound150Straw",
		STRAW180 = "multiRound180Straw",
		DRYGRASS_WINDROW125 = "multiRound125Hay",
		DRYGRASS_WINDROW150 = "multiRound150Hay",
		DRYGRASS_WINDROW180 = "multiRound180Hay",
	},
	MENU_ORDER = {"BLUE_GOLD", "GREEN", "RED_WHITE_BLUE", "DEFAULT", "WRAP_EDGE"},
	INPUT_CONTEXT = "BaleTextureChangerMenu",
	multiOptionTexts = {}, -- Where we store our translated texts
	modEnv = BaleTextureChanger.customEnvironment
}
local BaleTextureChangerMenu_mt = Class(BaleTextureChangerMenu, ScreenElement)

-- When the menu is loaded for the first time (or reloaded)
function BaleTextureChangerMenu.new(custom_mt)
	local self = ScreenElement.new(nil, custom_mt or BaleTextureChangerMenu_mt)

	self:registerControls(BaleTextureChangerMenu.CONTROLS)

	-- Make our texts (done this way so we can concatenate l10n entries, and use the texts to see what options are selected)
	if self.multiOptionTexts.BLUE_GOLD == nil then -- Don't try to do it again if it's already there (when reloading menu)
		self.multiOptionTexts.BLUE_GOLD = g_i18n:getText("ui_colorBlue", self.modEnv) .. " & " .. g_i18n:getText("gold", self.modEnv)
		self.multiOptionTexts.GREEN = g_i18n:getText("ui_colorGreen", self.modEnv)
		self.multiOptionTexts.RED_WHITE_BLUE = g_i18n:getText("ui_colorRed", self.modEnv) .. ", " .. g_i18n:getText("ui_colorWhite", self.modEnv) .. ", & " .. g_i18n:getText("ui_colorBlue", self.modEnv)
		self.multiOptionTexts.WRAP_EDGE = g_i18n:getText("wrapEdge", self.modEnv)
		self.multiOptionTexts.DEFAULT = g_i18n:getText("configuration_valueDefault", self.modEnv)
	end

	return self
end

-- When the menu opens, each time
function BaleTextureChangerMenu:onOpen()
	BaleTextureChangerMenu:superClass().onOpen(self)

	g_depthOfFieldManager:pushArea(0, 0, 1, 1) -- Blurry background!

	-- Compile multi-option texts into a single variable, since that's what the menu wants
	local texts = {}
	for i = 1, #self.MENU_ORDER do
		table.insert(texts, self.multiOptionTexts[self.MENU_ORDER[i]])
	end

	-- Put our option variable into each option box
	for _, val in pairs(self.CONTROLS) do
		self[val]:setTexts(texts)
	end
	
	-- Set the option boxes to show the current settings
	for key, val in pairs(self.CONTROLS) do
		local currentVal = BaleTextureChanger.currentSettings[key] -- Note that we're getting this from the not-menu code
		-- Look through the option box's texts, to find the right index
		for i = 1, #self[val].texts do
			if self[val].texts[i] == self.multiOptionTexts[currentVal] then
				-- Set that index as the active option
				self[val]:setState(i)
			end
		end
	end

	self:registerActionEvents() -- Make menu keys work
end

-- When the menu closes
function BaleTextureChangerMenu:onClose(element)
	BaleTextureChangerMenu:superClass().onClose(self)
	self:removeActionEvents()

	g_depthOfFieldManager:popArea() -- Remove the blur!
end

-- When the player clicks cancel or presses escape
function BaleTextureChangerMenu:onClickBack()
	BaleTextureChangerMenu:superClass().onClickBack(self)

	self:changeScreen(nil) -- Just close the menu
end

-- When the player confirms they want the settings saved
function BaleTextureChangerMenu:onClickApplyChanges()
	-- Loop through each option box, 
	local pendingChanges = {} -- Somewhere to store our list of changes required
	local changeCount = 0 -- Keeps count
	for keyBaleType, optionBox in pairs(self.CONTROLS) do -- Loop through each option box
		local optionIndex = self[optionBox]:getState() -- Grab the index that's active
		local optionText = self[optionBox].texts[optionIndex] -- Grab the text that's in that index
		-- Loop through translations, to find the bale option
		for keyOption, v in pairs (self.multiOptionTexts) do
			if v == optionText then
				-- Check if the settings are different
				if keyOption ~= BaleTextureChanger.currentSettings[keyBaleType] then
					-- Add any differences to our local variable
					pendingChanges[keyBaleType] = keyOption
					changeCount = changeCount + 1
				end
			end
		end
	end

	-- If there were no changes, we'll update all bales (so nil the table)
	if changeCount < 1 then
		pendingChanges = nil
	end
	
	-- Send our list of changes back to the main code
	BaleTextureChanger.applyChangesFromMenu(pendingChanges)
	
	-- Close the window
	self:changeScreen(nil)
end

-- Reloads the menu, for testing/dev
function BaleTextureChangerMenu:onClickReloadMenu()
		if g_gui.currentGuiName ~= nil and g_gui.currentGuiName ~= "" then
			local guiName = g_gui.currentGuiName
			local guiController = g_gui.currentGui.target

			g_gui:showGui("")
			g_i18n:load()

			g_gui.profiles = {}
			g_gui.traits = {}
			local success = g_gui:loadProfiles("dataS/guiProfiles.xml")
			g_gui:loadProfiles(BaleTextureChanger.modDir .. "gui/guiProfiles.xml")

			if not success then
				return "Failed to reload profiles"
			end

			local class = _G[self.modEnv][guiName]
			g_dummyGui = nil

			if class.createFromExistingGui ~= nil then
				g_dummyGui = class.createFromExistingGui(guiController, guiName)
			else
				g_dummyGui = class.new()

				g_gui.guis[guiName]:delete()
				g_gui.guis[guiName].target:delete()
				g_gui:loadGui(guiController.xmlFilename, guiName, g_dummyGui)
			end

			g_gui:showGui(guiName)

			Logging.warning("Reloaded gui " .. tostring(guiName))
		end

	return true
end


-- Just standard stuff below this line
function BaleTextureChangerMenu.createFromExistingGui(gui, guiName)
	local newGui = BaleTextureChangerMenu.new()

	g_gui:loadGui(gui.xmlFilename, guiName, newGui)

	return newGui
end

function BaleTextureChangerMenu:onGuiSetupFinished()
	BaleTextureChangerMenu:superClass().onGuiSetupFinished(self)
end

function BaleTextureChangerMenu:registerActionEvents()
	g_inputBinding:registerActionEvent(InputAction.AXIS_MTO_SCROLL, self, self.onInputScrollMTO, false, false, true, true)
end

function BaleTextureChangerMenu:removeActionEvents()
	g_inputBinding:removeActionEventsByTarget(self)
end
